package com.mh.web.controller.device;

import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.CommunicationParams;
import com.mh.common.core.domain.entity.SysDictData;
import com.mh.common.core.domain.vo.CollectionParamsManageVO2;
import com.mh.common.core.domain.vo.EnergyConsumptionVO;
import com.mh.common.core.domain.vo.EnergyQueryVO;
import com.mh.common.core.page.TableDataInfo;
import com.mh.common.utils.DateUtils;
import com.mh.common.utils.DictUtils;
import com.mh.common.utils.StringUtils;
import com.mh.common.utils.bean.BeanUtils;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.device.ICommunicationParamsService;
import com.mh.system.service.energy.IEnergyService;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 主机参数以及各个设备折线图数据
 * @date 2025-04-29 09:21:17
 */
@RestController
@RequestMapping("/chillers")
public class ChillersParamsController extends BaseController {

    private final ICollectionParamsManageService iCollectionParamsManageService;

    private final IEnergyService energyService;

    public ChillersParamsController(ICollectionParamsManageService iCollectionParamsManageService, IEnergyService energyService) {
        this.iCollectionParamsManageService = iCollectionParamsManageService;
        this.energyService = energyService;
    }

    /**
     * 获取注解全部参数内容数据
     *
     * @param collectionParamsManage
     * @return
     */
    @GetMapping("/list")
    public TableDataInfo list(CollectionParamsManage collectionParamsManage) {
        collectionParamsManage.setIsUse(0);
        List<CollectionParamsManage> list = iCollectionParamsManageService.selectCollectionParamsManageList(collectionParamsManage);
        // list中的CollectionParamsManage中的other_name去掉“号主机”之前数据
        list.forEach(item -> {
            String otherName = item.getOtherName();
            // 使用正则表达式去掉“号主机”及之前的内容
            String result = otherName != null ? otherName.replaceFirst(".*号主机", "") : otherName;
            result = result != null ? result.replace("-", "") : result;
            item.setOtherName(result);
        });
        // list中的CollectionParamsManage对象赋值到CollectionParamsManageVO2形成List<CollectionParamsManageVO2>
        List<CollectionParamsManageVO2> voList = list.stream()
                .filter(item -> item.getIsUse() != 1)
                .map(item -> {
                    CollectionParamsManageVO2 vo = new CollectionParamsManageVO2();
                    BeanUtils.copyProperties(item, vo);
                    vo.setCurValue(item.getCurValue().setScale(2).toString());
                    // 判断运行状态、启停、故障、本地远程
                    switch (vo.getParamType()) {
                        case "1":   // 运行状态
                            if (!StringUtils.isEmpty(vo.getCurValue())) {
                                vo.setCurValue(new BigDecimal(vo.getCurValue()).intValue() == 1 ? "运行" : "停止");
                            }
                            break;
                        case "2":   // 启停
                            if (!StringUtils.isEmpty(vo.getCurValue())) {
                                vo.setCurValue(new BigDecimal(vo.getCurValue()).intValue() == 1 ? "启动" : "停止");
                            }
                            break;
                        case "5":   // 故障
                            if (!StringUtils.isEmpty(vo.getCurValue())) {
                                vo.setCurValue(new BigDecimal(vo.getCurValue()).intValue() == 1 ? "故障" : "正常");
                            }
                            break;
                        case "6":   // 手自动切换
                            if (!StringUtils.isEmpty(vo.getCurValue())) {
                                vo.setCurValue(new BigDecimal(vo.getCurValue()).intValue() == 1 ? "手动" : "自动");
                            }
                            break;
                        case "22":   // 本地远程切换
                            if (!StringUtils.isEmpty(vo.getCurValue())) {
                                vo.setCurValue(new BigDecimal(vo.getCurValue()).intValue() == 1 ? "远程" : "本地");
                            }
                            break;
                        default:
                            break;
                    }
                    return vo;
                })
                .collect(Collectors.toList());

        // 再根据mtType进行分组
        Map<String, List<CollectionParamsManageVO2>> collect = voList.stream().collect(Collectors.groupingBy(CollectionParamsManageVO2::getMtType));
        // map转list
        List<SysDictData> sysMtTypeList = DictUtils.getDictCache("sys_mt_type");
        List<Map<String, Object>> result = collect.entrySet().stream().map(entry -> {
            Map<String, Object> map = new HashMap<>();
            map.put("mtType", entry.getKey());
            map.put("mtTypeName", sysMtTypeList.stream().filter(item -> item.getDictValue().equals(entry.getKey())).findFirst().get().getDictLabel());
            map.put("list", entry.getValue());
            return map;
        }).collect(Collectors.toList());
        result.sort((map1, map2) -> {
            Integer mtType1 = Integer.parseInt((String) map1.get("mtType"));
            Integer mtType2 = Integer.parseInt((String) map2.get("mtType"));
            return mtType1.compareTo(mtType2); // 升序
        });
        return getDataTable(result);
    }

    /**
     * 获取折线图数据:冷冻水进出水温度和冷却水进出水温度
     *
     * @param vo
     * @return
     */
    @PostMapping("/lineList")
    public AjaxResult lineList(@RequestBody EnergyQueryVO vo) {
        return energyService.chillerLine(vo);
    }

}
