package com.mh.web.controller.monitor;

import com.mh.common.core.controller.BaseController;
import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.dto.DeviceMonitorDTO;
import com.mh.common.core.domain.entity.DeviceLedger;
import com.mh.common.core.domain.entity.PolicyManage;
import com.mh.common.core.page.TableDataInfo;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.policy.IPolicyManageService;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 风柜监控系统
 * @date 2025-03-17 17:33:24
 */
@RestController
@RequestMapping("/device/ahu")
public class AirHandingUnitsMonitorController extends BaseController {

    private final ICollectionParamsManageService iCollectionParamsManageService;

    private final IPolicyManageService policyManageService;

    public AirHandingUnitsMonitorController(ICollectionParamsManageService iCollectionParamsManageService, IPolicyManageService policyManageService) {
        this.iCollectionParamsManageService = iCollectionParamsManageService;
        this.policyManageService = policyManageService;
    }

    /**
     * 获取监控列表内容数据
     */
    @GetMapping("/monitor/list")
    public TableDataInfo list(@RequestParam(name = "systemType") String systemType, @RequestParam(name = "houseId") String houseId)
    {
        List<DeviceMonitorDTO> list = iCollectionParamsManageService.selectMonitorListBySystemTypeAndHouseId(systemType, houseId);
        return getDataTable(list);
    }

    /**
     * 风柜获取定时开关机参数
     */
    @GetMapping("/monitor/timeList")
    public TableDataInfo timeList(@RequestParam(name = "systemType") String systemType,
                               @RequestParam(name = "funPolicyType") String funPolicyType,
                               @RequestParam(name = "houseId") String houseId)
    {
        List<?> list = policyManageService.selectDDCTimeList(systemType, funPolicyType, houseId);
        return getDataTable(list);
    }

    /**
     * 修改定时任务时间参数值
     */
    @PutMapping("/monitor/time")
    public AjaxResult edit(@RequestParam(name = "policyId") String policyId, @RequestParam(name = "timeValue") String timeValue)
    {
        return toAjax(policyManageService.updateTimeValue(policyId, timeValue));
    }

}
