package com.mh.framework.netty;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelOption;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class EchoServer {

    private final int port;

    public EchoServer(int port) {
        this.port = port;
    }

    public void start() {
        // 创建EventLoopGroup
        NioEventLoopGroup bossGroup = new NioEventLoopGroup();
        NioEventLoopGroup workerGroup = new NioEventLoopGroup();

        ServerBootstrap serverBootstrap = new ServerBootstrap();
        serverBootstrap.group(bossGroup, workerGroup)
                .channel(NioServerSocketChannel.class)
                .localAddress(port)
                .option(ChannelOption.SO_BACKLOG, 1204)
                .childHandler(new ServerChannelInitializer());

        // 异步绑定端口
        ChannelFuture channelFuture = serverBootstrap.bind();

        // 添加监听器处理绑定结果
        channelFuture.addListener(future -> {
            if (future.isSuccess()) {
                log.info("服务器启动成功，开始监听端口: {}", port);
            } else {
                log.error("服务器启动失败，端口: {}", port, future.cause());
                bossGroup.shutdownGracefully();   // 绑定失败立即关闭资源
                workerGroup.shutdownGracefully();
            }
        });

        // ❌ 移除 sync() 阻塞调用
        // channelFuture.channel().closeFuture().sync();  --> 删除这行

        // 可选：添加 JVM 关闭钩子优雅关闭资源
        Runtime.getRuntime().addShutdownHook(new Thread(() -> {
            log.info("JVM 正在关闭，准备释放 Netty 资源...");
            bossGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
            log.info("Netty 资源已释放");
        }));
    }

}
