package com.mh.framework.netty;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.mh.common.constant.Constants;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.SysDictData;
import com.mh.common.core.redis.RedisCache;
import com.mh.common.model.request.AdvantechDatas;
import com.mh.common.model.request.AdvantechReceiver;
import com.mh.common.utils.*;
import com.mh.common.utils.spring.SpringUtils;
import com.mh.framework.netty.session.ServerSession;
import com.mh.framework.netty.session.SessionMap;
import com.mh.framework.netty.task.CallbackTask;
//import com.mh.framework.netty.task.CallbackTaskScheduler;
import com.mh.framework.netty.task.CallbackTaskScheduler;
import com.mh.framework.rabbitmq.producer.SendMsgByTopic;
import com.mh.system.service.device.ICollectionParamsManageService;
import com.mh.system.service.device.IGatewayManageService;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import io.netty.util.ReferenceCountUtil;
import lombok.extern.slf4j.Slf4j;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
public class EchoServerHandler extends ChannelInboundHandlerAdapter {

    // 调用service层的接口信息
    IGatewayManageService gatewayManageService = SpringUtils.getBean(IGatewayManageService.class);
    ICollectionParamsManageService collectionParamsManageService = SpringUtils.getBean(ICollectionParamsManageService.class);
    SendMsgByTopic sendMsgByTopic = SpringUtils.getBean(SendMsgByTopic.class);
    RedisCache redisCache = SpringUtils.getBean(RedisCache.class);

    /**
     * 空闲次数
     */
    private int idleCount = 1;
    private int count = 0;
    private List<String> orderList;
    private int num = 0;
    private int size = 0;
    private String IP;
    private String port;
    private String receiveStr = "";
    private List<CollectionParamsManage> deviceCodeParamList;

    /**
     * 客户端连接会触发
     */
    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        log.info("Channel active......");
    }

    /**
     * 超时处理
     * 如果120秒没有接受客户端的心跳，就触发;
     * 如果超过3次，则直接关闭;
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object obj) throws Exception {
        if (obj instanceof IdleStateEvent) {
            IdleStateEvent event = (IdleStateEvent) obj;
            if (IdleState.READER_IDLE.equals(event.state())) {  //如果读通道处于空闲状态，说明没有接收到心跳命令
                log.info("第{}已经40秒没有接收到客户端的信息了", idleCount);
                receiveStr = "";
                num = num + 1;
                if (num > size - 1) {
                    num = 0;
//                    // 关闭连接
//                    ctx.close();
                    // 继续发送下一个采集指令
                    SendOrderUtils.sendAllOrder(deviceCodeParamList.get(num),ctx,num,size);
                } else {
                    // 继续发送下一个采集指令
                    SendOrderUtils.sendAllOrder(deviceCodeParamList.get(num), ctx, num, size);
                }
            }
        } else {
            super.userEventTriggered(ctx, obj);
        }
    }

    // 对于每一个传入的消息都要被调用
    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) {
        try {
            //接收到服务端发来的数据进行业务处理
            ByteBuf buf = (ByteBuf) msg;
            byte[] bytes = new byte[buf.readableBytes()];
            buf.readBytes(bytes);//复制内容到字节数组bytes
            buf.clear();
            // 截取IP地址
            IP = ExchangeStringUtil.getMidString(ctx.channel().remoteAddress() + "", "/", ":");
            // 截取端口号
            port = ExchangeStringUtil.getMidString(ctx.channel().remoteAddress() + "", ":", "");
            if (bytes.length <= 1024) {
                //将接收到的数据转为字符串，此字符串就是客户端发送的字符串
                receiveStr = receiveStr + ExchangeStringUtil.bytesToHexString(bytes);//将接收到的数据转为字符串，此字符串就是客户端发送的字符串
                receiveStr = receiveStr.replace("null", ""); //去null
                receiveStr = receiveStr.replace(" ", "");    //去空格
                //log.info("channelRead接收到的数据:" + receiveStr + ",length:" + receiveStr.length());
            }
        } catch (Exception e) {
            log.error("channelRead异常", e);
        } finally {
            ReferenceCountUtil.release(msg);
        }
    }

    // 当前批量读取中的最后一条消息
    @Override
    public void channelReadComplete(ChannelHandlerContext ctx) throws Exception {
        //心跳包报文: 24 00 60 95
        receiveStr = receiveStr.toUpperCase();//返回值全部变成大写
        log.info("channelReadComplete接收到的数据{}, 长度: ===> {}", receiveStr, receiveStr.length());
        //心跳包处理
        if ((receiveStr.length() == 8) && receiveStr.startsWith("24")) {
//        if ((receiveStr.length() == 8) && receiveStr.startsWith("C0A801FE")) {
            log.info("接收到心跳包 ===> {}", receiveStr);
            // 开始进行会话保存
            dealSession(ctx);
            idleCount = 1;
            port = receiveStr.substring(4, 8);//心跳包包含网关端口（自己定义返回心跳包）
            // 更新对应的网关在线情况
            gatewayManageService.updateGatewayStatus(receiveStr);
            //根据端口或者IP或者心跳包查询网关对应的项目名称
            // 生成采集指令
            if (!SpringUtils.getBean(RedisCache.class).hasKey(receiveStr)) {
                collectionParamsManageService.createDtuCollectionParams();
            }
            JSONArray arrayCache = SpringUtils.getBean(RedisCache.class).getCacheObject(receiveStr);
            if (StringUtils.isNotNull(arrayCache)) {
                deviceCodeParamList = arrayCache.toList(CollectionParamsManage.class);
            }
            size = deviceCodeParamList.size();
            log.info("deviceCodeParam size ===> {}", size);
            // 清空receiveStr
            receiveStr = "";
            num = 0;
            // 发送采集报文
            if (size > 0) {
                if (idleCount < 2) {
                    Thread.sleep(200);
                    SendOrderUtils.sendAllOrder(deviceCodeParamList.get(num), ctx, num, size);
                    idleCount++;
                } else {
                    ctx.channel().close();
                }
            } else {
                log.info("gateway not find deviceCodeParam!");
            }
        } else if (receiveStr.length() == 18) {
            // 水电表、热泵设置返回数据解析
            idleCount = 1;
            log.info("水电表、热泵设置接收==>{},长度:{}", receiveStr, receiveStr.length());
            nextSendOrder(ctx);
        } else if (receiveStr.length() == 12 || receiveStr.length() == 14) {
            // 热泵返回数据解析
            idleCount = 1;
            log.info("热泵读取接收===>{},长度:{}", receiveStr, receiveStr.length());
            nextSendOrder(ctx);
        } else if (receiveStr.length() == 16) {
            // 热泵设置指令返回
            // 判断是否有指令发送
            if (redisCache.hasKey("order_send") && redisCache.getCacheObject("order_send").equals(receiveStr)) {
                NettyTools.setReceiveMsg("order_wait", receiveStr);
            }
            nextSendOrder(ctx);
        } else if (receiveStr.length() > 50 && receiveStr.length() < 100) {
            idleCount = 1;
            // 清空receiveStr
            nextSendOrder(ctx);
        }
        ctx.flush();
    }

    private void dealSession(ChannelHandlerContext ctx) {
        // 获取表号
        String deviceCode =receiveStr;
        String meterNum = deviceCode;
        deviceCode = deviceCode + ctx.channel().remoteAddress();
        //新的session的创建
        ServerSession session = new ServerSession(ctx.channel(), deviceCode);

        //进行登录逻辑处理，异步进行处理。并且需要知道 处理的结果。 callbacktask就要
        //派上用场了
        String finalDeviceCode = deviceCode;
        CallbackTaskScheduler.add(new CallbackTask<Boolean>() {
            @Override
            public Boolean execute() throws Exception {
                //进行 login 逻辑的处理
                return action(session, finalDeviceCode, ctx);
            }
            //没有异常的话，我们进行处理
            @Override
            public void onBack(Boolean result) {
                if(result) {
                    log.info("设备保存会话： 设备号 = " + session.getSessionId());
                    //ctx.pipeline().remove(LoginRequestHandler.class); //压测需要放开
                } else {
                    log.info("设备刷新会话： 设备号 = " + session.getSessionId());
                    SessionMap.inst().updateSession(finalDeviceCode ,session, meterNum);
                    //log.info("设备登录失败： 设备号 = " + session.getSessionId());
                    //ServerSession.closeSession(ctx);
                    // 假如说已经在会话中了，直接断开连接
                    //ctx.close();
                }
            }
            //有异常的话，我们进行处理
            @Override
            public void onException(Throwable t) {
                log.info("设备登录异常： 设备号 = " + session.getSessionId());
                ServerSession.closeSession(ctx);
            }
        });
    }

    private void nextSendOrder(ChannelHandlerContext ctx) throws InterruptedException {
        // 发送指令响应不用解析
        if (receiveStr.length() != 16) {
            // 解析采集的报文，并保存到数据库
            analysisReceiveData(receiveStr, deviceCodeParamList.get(num));
        }
        // 清空receiveStr
        receiveStr = "";
        // 判断发送的下标，如果不等于指令数组大小
        num = num + 1;
        if (num > size - 1) {
            num = 0;
            Thread.sleep(1000);
            // 继续发送下一个采集指令
            SendOrderUtils.sendAllOrder(deviceCodeParamList.get(num), ctx, num, size);
            log.info("------一轮采集完成,继续下一轮--------");
        } else {
            // 添加一个状态值，判断是否继续发送指令 update by ljf on 2020-08-07
            if (Constants.WEB_FLAG) {
                num = 0;
                // 关闭连接
                receiveStr = null;
                ctx.close();
            } else {
                Thread.sleep(1000);
                // 继续发送下一个采集指令
                SendOrderUtils.sendAllOrder(deviceCodeParamList.get(num), ctx, num, size);
            }
        }
    }

    private void analysisReceiveData(final String receiveStr, final CollectionParamsManage deviceCodeParamEntity) {
        AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();
        String analysisData = "";
        switch (deviceCodeParamEntity.getParamType()) {
            case "16" ->
                // 电表
                    analysisData = analysisReceiveOrder485.analysisMeterOrder485(receiveStr, deviceCodeParamEntity);
            case "18" ->
                // 水表
                    analysisData = analysisReceiveOrder485.analysisWaterOrder485(receiveStr, deviceCodeParamEntity);
            case "5" ->
                // 热泵故障报警
                    analysisData = analysisReceiveOrder485.analysisHeatPumpOrder485(receiveStr, deviceCodeParamEntity);
            case "2" ->
                // 热泵启停控制
                    analysisData = analysisReceiveOrder485.analysisHeatPumpOrder485(receiveStr, deviceCodeParamEntity);
            case "12" ->
                // 热泵实际温度
                    analysisData = analysisReceiveOrder485.analysisHeatPumpOrder485(receiveStr, deviceCodeParamEntity);
            case "14" ->
                // 热泵读取温度设置
                    analysisData = analysisReceiveOrder485.analysisHeatPumpOrder485(receiveStr, deviceCodeParamEntity);
            default -> {
                log.info("设备类型错误");
                return;
            }
        }
        if (analysisData.isEmpty()) {
            log.info("解析数据为空");
            return;
        }
        // 格式化数据，配置成研华网关 AdvantechReceiver
        AdvantechReceiver advantechReceiver = new AdvantechReceiver();
        advantechReceiver.setTs(DateUtils.dateToString(new Date(), Constants.DATE_FORMAT));
        List<AdvantechDatas> advantechDatas = new ArrayList<>();
        AdvantechDatas datas = new AdvantechDatas();
        datas.setValue(new BigDecimal(analysisData));
        datas.setTag(deviceCodeParamEntity.getOtherName());
        advantechDatas.add(datas);
        advantechReceiver.setD(advantechDatas);
        sendMsgByTopic.sendToDeviceMQ(JSONObject.toJSONString(advantechReceiver));
    }

    // 异常捕捉
    @Override
    public void exceptionCaught(ChannelHandlerContext ctx, Throwable cause) {
        cause.getCause().printStackTrace();
        log.info("异常捕捉,执行ctx.close" + cause.getCause());
        ctx.close();                                      // 关闭该Channel
    }

    // 客户端断开
    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        ctx.close();// 关闭流
        log.info("客户端断开,执行ctx.close()......");
    }

    private boolean action(ServerSession session, String deviceCode, ChannelHandlerContext ctx) {
        //user验证
        boolean isValidUser = checkUser(deviceCode,session);
        session.bind();
        return true;
    }

    private boolean checkUser(String deviceCode,ServerSession session) {
        //当前用户已经登录
        if(SessionMap.inst().hasLogin(deviceCode)) {
            log.info("设备已经登录： 设备号 = " + deviceCode);
            return false;
        }
        //一般情况下，我们会将 user存储到 DB中，然后对user的用户名和密码进行校验
        //但是，我们这边没有进行db的集成，所以我们想一个别的办法进行user的校验。在我们的sessionMap进行以下校验
        //为什么选sessionmap，因为我们user的会话，都是存储到sessionmap中的，sessionmap中只要有这个user的会话，说明就是ok的
        return true;
    }

}
