package com.mh.framework.netty.session;

import io.netty.channel.Channel;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.util.AttributeKey;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

@Data
@Slf4j
public class ServerSession {
    public static final AttributeKey<ServerSession> SESSION_KEY =
            AttributeKey.valueOf("SESSION_KEY");
    //通道
    private Channel channel;
    private final String sessionId;
    private boolean isLogin = false;

    public ServerSession(Channel channel, String deviceCode){
        this.channel = channel;
        this.sessionId = deviceCode;
    }

    //session需要和通道进行一定的关联，他是在构造函数中关联上的；
    //session还需要通过sessionkey和channel进行再次的关联；channel.attr方法.set当前的
    // serverSession
    //session需要被添加到我们的SessionMap中
    public void bind(){
        log.info("server Session 会话进行绑定 ：" + channel.remoteAddress());
        channel.attr(SESSION_KEY).set(this);
        SessionMap.inst().addSession(sessionId, this);
        this.isLogin = true;
    }

    //通过channel获取session
    public static ServerSession getSession(ChannelHandlerContext ctx){
        Channel channel = ctx.channel();
        return channel.attr(SESSION_KEY).get();
    }

    //关闭session,新增返回一个meterNum用于纪录设备下线时间2024-05-08
    public static String closeSession(ChannelHandlerContext ctx){
        String meterNum = null;
        ServerSession serverSession = ctx.channel().attr(SESSION_KEY).get();
        if(serverSession != null && serverSession.getSessionId() != null) {
            ChannelFuture future = serverSession.channel.close();
            future.addListener((ChannelFutureListener) future1 -> {
                if(!future1.isSuccess()) {
                    log.info("Channel close error!");
                }
            });
            ctx.close();
            meterNum = serverSession.sessionId;
            SessionMap.inst().removeSession(serverSession.sessionId);
            log.info(ctx.channel().remoteAddress()+"  "+serverSession.sessionId + "==>移除会话");
        }
        return meterNum;
    }

    //写消息
    public void writeAndFlush(Object msg) {
        channel.writeAndFlush(msg);
    }
}
