package com.mh.framework.netty.session;

import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Data
@Slf4j
public class SessionMap {

    private ThreadLocal<Boolean> sceneThreadLocal = new ThreadLocal<>();

    //用单例模式进行sessionMap的创建
    private SessionMap(){}

    private static SessionMap singleInstance = new SessionMap();

    public static SessionMap inst() {
        return singleInstance;
    }

    //进行会话的保存
    //key 我们使用 sessionId；value 需要是 serverSession
    private ConcurrentHashMap<String, ServerSession> map = new ConcurrentHashMap<>(256);
    //添加session
    public void addSession(String sessionId, ServerSession s) {
        map.put(sessionId, s);
        log.info("IP地址:"+s.getChannel().remoteAddress()+"  "+ sessionId + " 表具上线,总共表具：" + map.size());
    }

    //删除session
    public void removeSession(String sessionId) {
        if(map.containsKey(sessionId)) {
            ServerSession s = map.get(sessionId);
            map.remove(sessionId);
            log.info("设备id下线:{},在线设备：{}", s.getSessionId(), map.size() );
        }
        return;
    }

    public boolean hasLogin(String sessionId) {
        Iterator<Map.Entry<String, ServerSession>> iterator = map.entrySet().iterator();
        while(iterator.hasNext()) {
            Map.Entry<String, ServerSession> next = iterator.next();
            if(sessionId != null && sessionId.equalsIgnoreCase(next.getValue().getSessionId())) {
                return true ;
            }
        }
        return false;
    }

    //如果在线，肯定有sessionMap里保存的 serverSession
    //如果不在线，serverSession也没有。用这个来判断是否在线
    public List<ServerSession> getSessionBy(String sessionId) {
        return map.values().stream().
                filter(s -> s.getSessionId().equals(sessionId)).
                collect(Collectors.toList());
    }

    public boolean getScene() {
        return sceneThreadLocal.get();
    }

    public void initScene(Boolean status) {
        if (sceneThreadLocal == null) {
            log.info("======创建ThreadLocal======");
            sceneThreadLocal = new ThreadLocal<>();
        }
        log.info("设置状态==>" + status);
        sceneThreadLocal.set(status);
    }

    public void clearScene() {
        initScene(null);
        sceneThreadLocal.remove();
    }

    public void updateSession(String sessionId, ServerSession session, String meterNum) {
        Iterator<Map.Entry<String, ServerSession>> iterator = map.entrySet().iterator();
        while(iterator.hasNext()) {
            Map.Entry<String, ServerSession> next = iterator.next();
            if (next.getKey().contains(meterNum)){
                iterator.remove();
            }
            if(sessionId != null && sessionId.equalsIgnoreCase(next.getValue().getSessionId())) {
                next.setValue(session);
            }
        }
    }

}
