package com.mh.framework.netty.task;

import com.google.common.util.concurrent.*;

import java.util.concurrent.Callable;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author LJF
 * @version 1.0
 * @project TAD_Server
 * @description 回调任务
 * @date 2023/7/3 15:34:11
 */
public class CallbackTaskScheduler extends Thread {
    private ConcurrentLinkedQueue<CallbackTask> executeTaskQueue =
            new ConcurrentLinkedQueue<>();
    private long sleepTime = 1000 * 10;
    private final ExecutorService pool = Executors.newCachedThreadPool();
    ListeningExecutorService lpool = MoreExecutors.listeningDecorator(pool);
    private static CallbackTaskScheduler inst = new CallbackTaskScheduler();
    private CallbackTaskScheduler() {
        this.start();
    }
    //add task
    public static <T> void add(CallbackTask<T> executeTask) {
        inst.executeTaskQueue.add(executeTask);
    }

    @Override
    public void run() {
        while (true) {
            handleTask();
            //为了避免频繁连接服务器,但是当前连接服务器过长导致失败
            //threadSleep(sleepTime);
        }
    }

    private void threadSleep(long sleepTime) {
        try {
            Thread.sleep(sleepTime);
        }catch (Exception e) {
            e.printStackTrace();
        }
    }

    //任务执行
    private void handleTask() {
        CallbackTask executeTask = null;
        while (executeTaskQueue.peek() != null) {
            executeTask = executeTaskQueue.poll();
            handleTask(executeTask);
        }
    }
    private <T> void handleTask(CallbackTask<T> executeTask) {
        ListenableFuture<T> future = lpool.submit(new Callable<T>() {
            public T call() throws Exception {
                return executeTask.execute();
            }
        });
        Futures.addCallback(future, new FutureCallback<T>() {
            @Override
            public void onSuccess(T t) {
                executeTask.onBack(t);
            }

            @Override
            public void onFailure(Throwable throwable) {
                executeTask.onException(throwable);
            }


        }, pool);
    }
}

